import Database from 'better-sqlite3';
import { migrate } from '@blackglory/better-sqlite3-migrations';

import utils from './utils.mjs';

// class

export class DB {
    constructor() {
        this.my_db = createDB();
    }
    siteInfo(site) {
        return getSiteInfo(this.my_db, site);
    }
    siteComments(site) {
        return getSiteComments(this.my_db, site);
    }
    pageComments(site, page) {
        return getPageComments(this.my_db, site, page);
    }
    insertPageComment(comment) {
        insertPageComment(this.my_db, comment);
    }
}

const db = new DB();
export default db;

// migrations

function migrations() {
    return [
    { version: 1,
      up: `
CREATE TABLE site (
  id integer primary key autoincrement,
  url text not null,
  comment_token text not null,
  length_limit integer
);

CREATE TABLE comment (
  id integer not null,
  site integer not null,
  path text not null,
  user text not null,
  user_website text,
  message text not null,
  published text default (datetime('now')),
  reply_to integer,
  FOREIGN KEY(site) REFERENCES site(id),
  PRIMARY KEY (site, path, id)
);
      `,
      down: `
DROP TABLE comment;
DROP TABLE site;
      `
    }
    ];
}

// setup

function createDB() {
    const db = new Database(process.env.DB || utils.DEFAULT_DB_PATH);
    db.pragma('journal_mode = WAL');
    migrate(db, migrations(), 1);
    return db;
}

// queries

function getSiteInfo(db, site) {
    return db.prepare(`SELECT comment_token, length_limit as message_length_limit FROM site WHERE url = ?`).get(site);
}

function insertPageComment(db, object) {
    const stmt = db.prepare(`
        INSERT INTO comment(id, site, path, user, user_website, message, reply_to)
        SELECT
          ( SELECT count(*)
            FROM (SELECT * FROM comment WHERE path = @path) c
            JOIN (SELECT id FROM site WHERE url = @site) s
            ON s.id = c.site
          ),
          ( SELECT id FROM site WHERE url = @site ),
            @path,
            @user,
            @user_website,
            @message,
            @reply_to
        RETURNING
          id as id,
          user,
          user_website,
          message,
          published,
          reply_to
        ;
    `);
    return stmt.all(object);
}

function getPageComments(db, site, path) {
    const stmt = db.prepare(`
        SELECT
          c.id as id,
          user,
          user_website,
          message,
          published,
          reply_to
        FROM
          (SELECT id from site where url = @site) s
        JOIN (SELECT * FROM comment WHERE path = @path) c
        ON c.site = s.id
        ORDER BY c.id
    ;`);
    return stmt.all({ site, path });
}

function getSiteComments(db, site) {
    const stmt = db.prepare(`
        SELECT
          c.id,
          s.url as site,
          c.path,
          c.user,
          c.user_website,
          c.message,
          c.published
        FROM
          (SELECT id, url from site where url = @site) s
        JOIN comment c
        ON c.site = s.id
        ORDER BY c.published DESC
        ;
    `);
    return stmt.all({ site });
}
